﻿using GrapeCity.CalcEngine;
using GrapeCity.Spreadsheet;

namespace SpreadNETGetLinks
{
    /// <summary>
    /// Spread.NET Calculation Function GETLINKS() implementation
    /// </summary>
    /// <remarks>
    /// This function gets the list of external links in the workbook and returns it in a vertical array.
    /// </remarks>
    sealed class GetLinksFunction : Function
    {
        public GetLinksFunction() : base("GETLINKS", 0, 0, FunctionAttributes.Array)
        {
        }
        private IWorkbook GetWorkbook(IEvaluationContext context)
        {   // use IEvaluationContext.ReferenceSource to get the IWorkbook
            IWorkbook workbook;
            IWorksheet worksheet = context.ReferenceSource as IWorksheet;
            if (worksheet != null)
            {
                workbook = worksheet.Workbook;
            }
            else
            {
                workbook = context.ReferenceSource as IWorkbook;
            }
            return workbook;
        }
        protected override void Evaluate(IArguments args, IValue result)
        {
            IWorkbook workbook = GetWorkbook(args.EvaluationContext);
            if (workbook != null && workbook.LinkSources != null && workbook.LinkSources.Length > 0)
            {   // copy the link sources to array and return
                string[] links = workbook.LinkSources;
                IValuesArray array = result.CreateValuesArray(links.Length, 1);
                for (int i = 0; i < links.Length; i++)
                    array[i, 0].SetValue(links[i]);
                result.SetValue(array);
            }
            else // return #N/A error
                result.SetValue(CalcError.NotAvailable);
        }
    }
}
